@@ -303,7 +303,7 @@ on_button_press (GooCanvasItem *item,
 	  fleur = gdk_cursor_new (GDK_FLEUR);
 	  canvas = goo_canvas_item_get_canvas (item);
 	  goo_canvas_pointer_grab (canvas, item,
-				   GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+				   GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_RELEASE_MASK,
 				   fleur,
 				   event->time);
 	  gdk_cursor_unref (fleur);
@@ -370,22 +370,23 @@ on_item_created (GooCanvas          *canvas,
   if (!goo_canvas_item_model_get_parent (model))
     {
       g_signal_connect (item, "button_press_event",
-			(GtkSignalFunc) on_background_button_press, NULL);
+			G_CALLBACK (on_background_button_press), NULL);
     }
   else if (!g_object_get_data (G_OBJECT (model), "skip-signal-connection"))
     {
       g_signal_connect (item, "motion_notify_event",
-			(GtkSignalFunc) on_motion_notify, NULL);
+			G_CALLBACK (on_motion_notify), NULL);
       g_signal_connect (item, "button_press_event",
-			(GtkSignalFunc) on_button_press, NULL);
+			G_CALLBACK (on_button_press), NULL);
       g_signal_connect (item, "button_release_event",
-			(GtkSignalFunc) on_button_release, NULL);
+			G_CALLBACK (on_button_release), NULL);
     }
 }
 
 
 GtkWidget *
-create_canvas_primitives (GooCanvasItemModel *model)
+create_canvas_primitives (GooCanvasItemModel *model,
+			  GooCanvasItemModel *static_model)
 {
 	GtkWidget *vbox;
 	GtkWidget *hbox;
@@ -414,11 +415,14 @@ create_canvas_primitives (GooCanvasItemModel *model)
 		      "bounds-from-origin", FALSE,
 		      "bounds-padding", 4.0,
 		      "background-color-rgb", 0xC3C3FF,
+#if 0
+		      "redraw-when-scrolled", TRUE,
+#endif
 		      NULL);
 	goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 604, 454);
 
 	g_signal_connect (canvas, "item_created",
-			  (GtkSignalFunc) on_item_created, NULL);
+			  G_CALLBACK (on_item_created), NULL);
 
 	/* Zoom */
 
@@ -429,7 +433,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	adj = GTK_ADJUSTMENT (gtk_adjustment_new (1.00, 0.05, 50.00, 0.05, 0.50, 0.50));
 	w = gtk_spin_button_new (adj, 0.0, 2);
 	g_signal_connect (adj, "value_changed",
-			  (GtkSignalFunc) zoom_changed,
+			  G_CALLBACK (zoom_changed),
 			  canvas);
 	gtk_widget_set_size_request (w, 50, -1);
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
@@ -441,7 +445,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	/*gtk_widget_show (w);*/
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) center_toggled,
+			  G_CALLBACK (center_toggled),
 			  canvas);
 
 	/* Move Ellipse */
@@ -450,7 +454,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) move_ellipse_clicked,
+			  G_CALLBACK  (move_ellipse_clicked),
 			  canvas);
 
 	/* Animate Ellipse */
@@ -459,7 +463,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) animate_ellipse_clicked,
+			  G_CALLBACK (animate_ellipse_clicked),
 			  canvas);
 	
 	/* Stop Animation */
@@ -468,7 +472,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) stop_animation_clicked,
+			  G_CALLBACK (stop_animation_clicked),
 			  canvas);
 	
 
@@ -479,7 +483,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) write_pdf_clicked,
+			  G_CALLBACK (write_pdf_clicked),
 			  canvas);
 #endif
 
@@ -487,7 +491,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) change_bounds_clicked,
+			  G_CALLBACK (change_bounds_clicked),
 			  canvas);
 
 	hbox = gtk_hbox_new (FALSE, 4);
@@ -504,21 +508,21 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) scroll_to_50_50_clicked,
+			  G_CALLBACK (scroll_to_50_50_clicked),
 			  canvas);
 
 	w = gtk_button_new_with_label("250,250");
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) scroll_to_250_250_clicked,
+			  G_CALLBACK (scroll_to_250_250_clicked),
 			  canvas);
 
 	w = gtk_button_new_with_label("500,500");
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "clicked",
-			  (GtkSignalFunc) scroll_to_500_500_clicked,
+			  G_CALLBACK (scroll_to_500_500_clicked),
 			  canvas);
 
 	/* Scroll anchor */
@@ -532,7 +536,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_NW));
@@ -542,7 +546,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_N));
@@ -552,7 +556,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_NE));
@@ -562,7 +566,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_W));
@@ -572,7 +576,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_CENTER));
@@ -582,7 +586,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_E));
@@ -592,7 +596,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_SW));
@@ -602,7 +606,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_S));
@@ -612,7 +616,7 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
 	gtk_widget_show (w);
 	g_signal_connect (w, "toggled",
-			  (GtkSignalFunc) anchor_toggled,
+			  G_CALLBACK (anchor_toggled),
 			  canvas);
 	g_object_set_data (G_OBJECT (w), "anchor",
 			   GINT_TO_POINTER (GTK_ANCHOR_SE));
@@ -631,9 +635,11 @@ create_canvas_primitives (GooCanvasItemModel *model)
 	   item views to render each of the items. */
 	goo_canvas_set_root_item_model (GOO_CANVAS (canvas), model);
 
+	goo_canvas_set_static_root_item_model (GOO_CANVAS (canvas), static_model);
+
 #if 0
 	g_signal_connect_after (canvas, "key_press_event",
-				(GtkSignalFunc) key_press,
+				G_CALLBACK (key_press),
 				NULL);
 
 	GTK_WIDGET_SET_FLAGS (canvas, GTK_CAN_FOCUS);
@@ -1211,6 +1217,104 @@ setup_images (GooCanvasItemModel *root)
 }
 
 
+static GooCanvasItemModel*
+create_static_model ()
+{
+  GooCanvasItemModel *static_root, *group, *item;
+
+  static_root = goo_canvas_group_model_new (NULL, NULL);
+
+  group = goo_canvas_group_model_new (static_root, NULL);
+
+  item = goo_canvas_polyline_model_new_line (group,
+					     40.0, 410.0,
+					     40.0, 330.0,
+					     "stroke-color", "midnightblue",
+					     "line-width", 3.0,
+					     "end-arrow", TRUE,
+					     "arrow-tip-length", 3.0,
+					     "arrow-length", 4.0,
+					     "arrow-width", 3.5,
+					     NULL);
+
+  item = goo_canvas_polyline_model_new_line (group,
+					     32.0, 370.0,
+					     48.0, 370.0,
+					     "stroke-color", "midnightblue",
+					     "line-width", 3.0,
+					     NULL);
+
+  item = goo_canvas_text_model_new (group, "N", 40, 320, -1, GTK_ANCHOR_S,
+				    "font", "Sans 12",
+				    NULL);
+
+  return static_root;
+}
+
+
+static void
+test_simple_transforms (GooCanvasItemModel *root)
+{
+#if 0
+  GooCanvasItemModel *item;
+  int rotation, x, y;
+  double scale, x_out, y_out, scale_out, rotation_out, max_error = 0.0000001;
+
+  item = goo_canvas_rect_model_new (root, 20, 30, 50, 30,
+				    NULL);
+
+  for (rotation = 0; rotation < 360; rotation += 20)
+    {
+      scale = 0.1;
+      while (scale < 10)
+	{
+	  g_print ("Rotation: %i Scale: %g\n", rotation, scale);
+
+	  for (x = -100; x < 100; x += 10)
+	    {
+	      for (y = -100; y < 100; y += 10)
+		{
+		  goo_canvas_item_model_set_simple_transform (item, x, y, scale,
+							      rotation);
+
+		  goo_canvas_item_model_get_simple_transform (item, &x_out, &y_out,
+							      &scale_out,
+							      &rotation_out);
+
+		  if (fabs (x_out - x) > max_error
+		      || fabs (y_out - y) > max_error
+		      || fabs (scale_out - scale) > max_error
+		      || fabs (rotation_out - rotation) > max_error)
+		    {
+		      g_print ("IN X:%i Y:%i Scale:%g Rotation:%i -> %g, %g, %g, %g\n", x, y, scale, rotation, x_out, y_out, scale_out, rotation_out);
+		    }
+		}
+	    }
+	  scale += 0.1;
+	}
+    }
+#endif
+}
+
+
+static void
+setup_grids (GooCanvasItemModel *root)
+{
+  GooCanvasItemModel *item;
+
+  item = goo_canvas_grid_model_new (root, 80, 310, 90, 90, 10, 10, 5, 5,
+				    "stroke-color", "yellow",
+				    "fill-color", "pink",
+				    "border-width", 2.0,
+				    "border-color", "red",
+				    "vert-grid-line-color", "lightblue",
+				    "horz-grid-line-width", 1.0,
+				    "vert-grid-line-width", 1.0,
+				    "vert-grid-lines-on-top", TRUE,
+				    NULL);
+}
+
+
 static GooCanvasItemModel*
 create_model (void)
 {
@@ -1228,11 +1332,14 @@ create_model (void)
   setup_texts (root); 
   setup_images (root);
   setup_invisible_texts (root);
+  setup_grids (root);
 #endif
 #if 0
   setup_widgets (root);
 #endif
 
+  test_simple_transforms (root);
+
   return root;
 }
 
@@ -1248,14 +1355,15 @@ on_delete_event (GtkWidget *window,
 
 
 static GtkWidget*
-create_window (GooCanvasItemModel *model)
+create_window (GooCanvasItemModel *model,
+	       GooCanvasItemModel *static_model)
 {
   GtkWidget *window, *notebook;
 
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_default_size (GTK_WINDOW (window), 640, 600);
   gtk_widget_show (window);
-  g_signal_connect (window, "delete_event", (GtkSignalFunc) on_delete_event,
+  g_signal_connect (window, "delete_event", G_CALLBACK (on_delete_event),
 		    NULL);
 
   notebook = gtk_notebook_new ();
@@ -1263,7 +1371,7 @@ create_window (GooCanvasItemModel *model)
   gtk_container_add (GTK_CONTAINER (window), notebook);
 
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
-			    create_canvas_primitives (model),
+			    create_canvas_primitives (model, static_model),
 			    gtk_label_new ("Primitives"));
 #if 1
   gtk_notebook_append_page (GTK_NOTEBOOK (notebook),
@@ -1328,21 +1436,23 @@ create_window (GooCanvasItemModel *model)
 int
 main (int argc, char *argv[])
 {
-  GooCanvasItemModel *model;
+  GooCanvasItemModel *model, *static_model;
   GtkWidget *window;
 
   gtk_set_locale ();
   gtk_init (&argc, &argv);
 
   model = create_model ();
+  static_model = create_static_model ();
 
   /* Create 2 windows to show off multiple views. */
-  window = create_window (model);
+  window = create_window (model, static_model);
 #if 1
-  window = create_window (model);
+  window = create_window (model, static_model);
 #endif
 
   g_object_unref (model);
+  g_object_unref (static_model);
 
   gtk_main ();
   return 0;
