@@ -22,7 +22,7 @@
  * method. (#GooCanvasEllipse, #GooCanvasRect and #GooCanvasPath do this.)
  *
  * More complicated items need to implement the update(), paint() and
- * is_item_at() methods. (#GooCanvasImage, #GooCanvasPolyline,
+ * is_item_at() methods instead. (#GooCanvasImage, #GooCanvasPolyline,
  * #GooCanvasText and #GooCanvasWidget do this.) They may also need to
  * override some of the other GooCanvasItem methods such as set_canvas(),
  * set_parent() or allocate_area() if special code is needed. (#GooCanvasWidget
@@ -96,6 +96,20 @@ static void goo_canvas_item_simple_set_property (GObject              *object,
 						 const GValue         *value,
 						 GParamSpec           *pspec);
 
+static void     goo_canvas_item_simple_default_create_path (GooCanvasItemSimple   *simple,
+							    cairo_t               *cr);
+static void     goo_canvas_item_simple_default_update      (GooCanvasItemSimple   *simple,
+							    cairo_t               *cr);
+static void     goo_canvas_item_simple_default_paint       (GooCanvasItemSimple   *simple,
+							    cairo_t               *cr,
+							    const GooCanvasBounds *bounds);
+static gboolean goo_canvas_item_simple_default_is_item_at  (GooCanvasItemSimple   *simple,
+							    double                 x,
+							    double                 y,
+							    cairo_t               *cr,
+							    gboolean               is_pointer_event);
+
+
 G_DEFINE_TYPE_WITH_CODE (GooCanvasItemSimple, goo_canvas_item_simple,
 			 G_TYPE_OBJECT,
 			 G_IMPLEMENT_INTERFACE (GOO_TYPE_CANVAS_ITEM,
@@ -218,7 +232,7 @@ goo_canvas_item_simple_install_common_properties (GObjectClass *gobject_class)
 						      _("Stroke Color RGBA"),
 						      _("The color to use for the item's perimeter, specified as a 32-bit integer value. To disable painting set the 'stroke-pattern' property to NULL"),
 						      0, G_MAXUINT, 0,
-						      G_PARAM_WRITABLE));
+						      G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_STROKE_PIXBUF,
                                    g_param_spec_object ("stroke-pixbuf",
@@ -239,7 +253,7 @@ goo_canvas_item_simple_install_common_properties (GObjectClass *gobject_class)
 						      _("Fill Color RGBA"),
 						      _("The color to use to paint the interior of the item, specified as a 32-bit integer value. To disable painting set the 'fill-pattern' property to NULL"),
 						      0, G_MAXUINT, 0,
-						      G_PARAM_WRITABLE));
+						      G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_FILL_PIXBUF,
                                    g_param_spec_object ("fill-pixbuf",
@@ -328,6 +342,11 @@ goo_canvas_item_simple_class_init (GooCanvasItemSimpleClass *klass)
     }
 
   goo_canvas_item_simple_install_common_properties (gobject_class);
+
+  klass->simple_create_path = goo_canvas_item_simple_default_create_path;
+  klass->simple_update      = goo_canvas_item_simple_default_update;
+  klass->simple_paint       = goo_canvas_item_simple_default_paint;
+  klass->simple_is_item_at  = goo_canvas_item_simple_default_is_item_at;
 }
 
 
@@ -493,6 +512,20 @@ goo_canvas_item_simple_get_common_property (GObject                 *object,
       g_value_set_enum (value, svalue ? svalue->data[0].v_long : CAIRO_HINT_METRICS_OFF);
       break;
 
+      /* Convenience properties. */
+    case PROP_STROKE_COLOR_RGBA:
+      svalue = goo_canvas_style_get_property (style, goo_canvas_style_stroke_pattern_id);
+      if (svalue)
+	goo_canvas_get_rgba_value_from_pattern (svalue->data[0].v_pointer,
+						value);
+      break;
+    case PROP_FILL_COLOR_RGBA:
+      svalue = goo_canvas_style_get_property (style, goo_canvas_style_fill_pattern_id);
+      if (svalue)
+	goo_canvas_get_rgba_value_from_pattern (svalue->data[0].v_pointer,
+						value);
+      break;
+
       /* Other properties. */
     case PROP_TRANSFORM:
       g_value_set_boxed (value, simple_data->transform);
@@ -559,10 +592,6 @@ goo_canvas_item_simple_set_common_property (GObject                 *object,
 					    GParamSpec              *pspec)
 {
   GooCanvasStyle *style;
-  GdkColor color = { 0, 0, 0, 0, };
-  guint rgba, red, green, blue, alpha;
-  GdkPixbuf *pixbuf;
-  cairo_surface_t *surface;
   cairo_pattern_t *pattern;
   gboolean recompute_bounds = FALSE;
   cairo_matrix_t *transform;
@@ -651,74 +680,29 @@ goo_canvas_item_simple_set_common_property (GObject                 *object,
 
       /* Convenience properties. */
     case PROP_STROKE_COLOR:
-      if (g_value_get_string (value))
-	gdk_color_parse (g_value_get_string (value), &color);
-      pattern = cairo_pattern_create_rgb (color.red / 65535.0,
-					  color.green / 65535.0,
-					  color.blue / 65535.0);
-      g_value_init (&tmpval, GOO_TYPE_CAIRO_PATTERN);
-      g_value_take_boxed (&tmpval, pattern);
-      goo_canvas_style_set_property (style, goo_canvas_style_stroke_pattern_id, &tmpval);
-      g_value_unset (&tmpval);
+      pattern = goo_canvas_create_pattern_from_color_value (value);
+      goo_canvas_set_style_property_from_pattern (style, goo_canvas_style_stroke_pattern_id, pattern);
       break;
     case PROP_STROKE_COLOR_RGBA:
-      rgba = g_value_get_uint (value);
-      red   = (rgba >> 24) & 0xFF;
-      green = (rgba >> 16) & 0xFF;
-      blue  = (rgba >> 8)  & 0xFF;
-      alpha = (rgba)       & 0xFF;
-      pattern = cairo_pattern_create_rgba (red / 255.0, green / 255.0,
-					   blue / 255.0, alpha / 255.0);
-      g_value_init (&tmpval, GOO_TYPE_CAIRO_PATTERN);
-      g_value_take_boxed (&tmpval, pattern);
-      goo_canvas_style_set_property (style, goo_canvas_style_stroke_pattern_id, &tmpval);
-      g_value_unset (&tmpval);
+      pattern = goo_canvas_create_pattern_from_rgba_value (value);
+      goo_canvas_set_style_property_from_pattern (style, goo_canvas_style_stroke_pattern_id, pattern);
       break;
     case PROP_STROKE_PIXBUF:
-      pixbuf = g_value_get_object (value);
-      surface = goo_canvas_cairo_surface_from_pixbuf (pixbuf);
-      pattern = cairo_pattern_create_for_surface (surface);
-      cairo_surface_destroy (surface);
-      cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
-      g_value_init (&tmpval, GOO_TYPE_CAIRO_PATTERN);
-      g_value_take_boxed (&tmpval, pattern);
-      goo_canvas_style_set_property (style, goo_canvas_style_stroke_pattern_id, &tmpval);
-      g_value_unset (&tmpval);
+      pattern = goo_canvas_create_pattern_from_pixbuf_value (value);
+      goo_canvas_set_style_property_from_pattern (style, goo_canvas_style_stroke_pattern_id, pattern);
       break;
+
     case PROP_FILL_COLOR:
-      if (g_value_get_string (value))
-	gdk_color_parse (g_value_get_string (value), &color);
-      pattern = cairo_pattern_create_rgb (color.red / 65535.0,
-					  color.green / 65535.0,
-					  color.blue / 65535.0);
-      g_value_init (&tmpval, GOO_TYPE_CAIRO_PATTERN);
-      g_value_take_boxed (&tmpval, pattern);
-      goo_canvas_style_set_property (style, goo_canvas_style_fill_pattern_id, &tmpval);
-      g_value_unset (&tmpval);
+      pattern = goo_canvas_create_pattern_from_color_value (value);
+      goo_canvas_set_style_property_from_pattern (style, goo_canvas_style_fill_pattern_id, pattern);
       break;
     case PROP_FILL_COLOR_RGBA:
-      rgba = g_value_get_uint (value);
-      red   = (rgba >> 24) & 0xFF;
-      green = (rgba >> 16) & 0xFF;
-      blue  = (rgba >> 8)  & 0xFF;
-      alpha = (rgba)       & 0xFF;
-      pattern = cairo_pattern_create_rgba (red / 255.0, green / 255.0,
-					   blue / 255.0, alpha / 255.0);
-      g_value_init (&tmpval, GOO_TYPE_CAIRO_PATTERN);
-      g_value_take_boxed (&tmpval, pattern);
-      goo_canvas_style_set_property (style, goo_canvas_style_fill_pattern_id, &tmpval);
-      g_value_unset (&tmpval);
+      pattern = goo_canvas_create_pattern_from_rgba_value (value);
+      goo_canvas_set_style_property_from_pattern (style, goo_canvas_style_fill_pattern_id, pattern);
       break;
     case PROP_FILL_PIXBUF:
-      pixbuf = g_value_get_object (value);
-      surface = goo_canvas_cairo_surface_from_pixbuf (pixbuf);
-      pattern = cairo_pattern_create_for_surface (surface);
-      cairo_surface_destroy (surface);
-      cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
-      g_value_init (&tmpval, GOO_TYPE_CAIRO_PATTERN);
-      g_value_take_boxed (&tmpval, pattern);
-      goo_canvas_style_set_property (style, goo_canvas_style_fill_pattern_id, &tmpval);
-      g_value_unset (&tmpval);
+      pattern = goo_canvas_create_pattern_from_pixbuf_value (value);
+      goo_canvas_set_style_property_from_pattern (style, goo_canvas_style_fill_pattern_id, pattern);
       break;
 
       /* Other properties. */
@@ -771,7 +755,6 @@ goo_canvas_item_simple_set_property (GObject              *object,
   GooCanvasItem *parent;
   AtkObject *accessible;
   gboolean recompute_bounds;
-  gint child_num;
 
   if (simple->model)
     {
@@ -783,11 +766,7 @@ goo_canvas_item_simple_set_property (GObject              *object,
     {
     case PROP_PARENT:
       parent = g_value_get_object (value);
-      if (simple->parent)
-	{
-	  child_num = goo_canvas_item_find_child (simple->parent, item);
-	  goo_canvas_item_remove_child (simple->parent, child_num);
-	}
+      goo_canvas_item_remove (item);
       goo_canvas_item_add_child (parent, item, -1);
       break;
     case PROP_TITLE:
@@ -864,6 +843,9 @@ void
 goo_canvas_item_simple_changed    (GooCanvasItemSimple *item,
 				   gboolean             recompute_bounds)
 {
+  GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
+  GooCanvasItemSimpleData *simple_data = simple->simple_data;
+
   if (recompute_bounds)
     {
       item->need_entire_subtree_update = TRUE;
@@ -879,7 +861,7 @@ goo_canvas_item_simple_changed    (GooCanvasItemSimple *item,
   else
     {
       if (item->canvas)
-	goo_canvas_request_redraw (item->canvas, &item->bounds);
+	goo_canvas_request_item_redraw (item->canvas, &item->bounds, simple_data->is_static);
     }
 }
 
@@ -983,7 +965,6 @@ goo_canvas_item_simple_get_items_at (GooCanvasItem  *item,
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
   GooCanvasItemSimpleData *simple_data = simple->simple_data;
   double user_x = x, user_y = y;
-  GooCanvasPointerEvents pointer_events = GOO_CANVAS_EVENTS_ALL;
   cairo_matrix_t matrix;
   gboolean add_item = FALSE;
 
@@ -1006,8 +987,6 @@ goo_canvas_item_simple_get_items_at (GooCanvasItem  *item,
 	      || (simple_data->visibility == GOO_CANVAS_ITEM_VISIBLE_ABOVE_THRESHOLD
 		  && simple->canvas->scale < simple_data->visibility_threshold)))
 	return found_items;
-
-      pointer_events = simple_data->pointer_events;
     }
 
   cairo_save (cr);
@@ -1033,20 +1012,8 @@ goo_canvas_item_simple_get_items_at (GooCanvasItem  *item,
 	}
     }
 
-  if (class->simple_is_item_at)
-    {
-      add_item = class->simple_is_item_at (simple, user_x, user_y, cr,
-					   is_pointer_event);
-    }
-  else
-    {
-      /* Use the virtual method subclasses define to create the path. */
-      class->simple_create_path (simple, cr);
-
-      if (goo_canvas_item_simple_check_in_path (simple, user_x, user_y, cr,
-						pointer_events))
-	add_item = TRUE;
-    }
+  add_item = class->simple_is_item_at (simple, user_x, user_y, cr,
+                                       is_pointer_event);
 
   cairo_restore (cr);
 
@@ -1057,6 +1024,31 @@ goo_canvas_item_simple_get_items_at (GooCanvasItem  *item,
 }
 
 
+static gboolean
+goo_canvas_item_simple_default_is_item_at (GooCanvasItemSimple *simple,
+					   double               x,
+					   double               y,
+					   cairo_t             *cr,
+					   gboolean             is_pointer_event)
+{
+  GooCanvasItemSimpleClass *class = GOO_CANVAS_ITEM_SIMPLE_GET_CLASS (simple);
+
+  GooCanvasItemSimpleData *simple_data = simple->simple_data;
+  GooCanvasPointerEvents pointer_events = GOO_CANVAS_EVENTS_ALL;
+
+  if (is_pointer_event)
+    pointer_events = simple_data->pointer_events;
+
+  /* Use the virtual method subclasses define to create the path. */
+  class->simple_create_path (simple, cr);
+
+  if (goo_canvas_item_simple_check_in_path (simple, x, y, cr, pointer_events))
+    return TRUE;
+
+  return FALSE;
+}
+
+
 static gboolean
 goo_canvas_item_simple_is_visible  (GooCanvasItem   *item)
 {
@@ -1075,6 +1067,27 @@ goo_canvas_item_simple_is_visible  (GooCanvasItem   *item)
 }
 
 
+static gboolean
+goo_canvas_item_simple_get_is_static  (GooCanvasItem   *item)
+{
+  GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
+  GooCanvasItemSimpleData *simple_data = simple->simple_data;
+
+  return simple_data->is_static;
+}
+
+
+static void
+goo_canvas_item_simple_set_is_static  (GooCanvasItem   *item,
+				       gboolean         is_static)
+{
+  GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
+  GooCanvasItemSimpleData *simple_data = simple->simple_data;
+
+  simple_data->is_static = is_static;
+}
+
+
 /**
  * goo_canvas_item_simple_check_style:
  * @item: a #GooCanvasItemSimple.
@@ -1129,17 +1142,7 @@ goo_canvas_item_simple_update_internal  (GooCanvasItemSimple *simple,
 
   cairo_get_matrix (cr, &transform);
 
-  if (class->simple_update)
-    {
-      class->simple_update (simple, cr);
-    }
-  else
-    {
-      /* Use the identity matrix to get the bounds completely in user space. */
-      cairo_identity_matrix (cr);
-      class->simple_create_path (simple, cr);
-      goo_canvas_item_simple_get_path_bounds (simple, cr, &simple->bounds);
-    }
+  class->simple_update (simple, cr);
 
   /* Modify the extents by the item's clip path. */
   if (simple_data->clip_path_commands)
@@ -1164,6 +1167,20 @@ goo_canvas_item_simple_update_internal  (GooCanvasItemSimple *simple,
 }
 
 
+static void
+goo_canvas_item_simple_default_update (GooCanvasItemSimple   *simple,
+				       cairo_t               *cr)
+{
+  GooCanvasItemSimpleClass *class = GOO_CANVAS_ITEM_SIMPLE_GET_CLASS (simple);
+
+  /* Use the identity matrix to get the bounds completely in user space. */
+  cairo_identity_matrix (cr);
+
+  class->simple_create_path (simple, cr);
+  goo_canvas_item_simple_get_path_bounds (simple, cr, &simple->bounds);
+}
+
+
 static void
 goo_canvas_item_simple_update  (GooCanvasItem   *item,
 				gboolean         entire_tree,
@@ -1178,7 +1195,7 @@ goo_canvas_item_simple_update  (GooCanvasItem   *item,
   if (entire_tree || simple->need_update)
     {
       /* Request a redraw of the existing bounds. */
-      goo_canvas_request_redraw (simple->canvas, &simple->bounds);
+      goo_canvas_request_item_redraw (simple->canvas, &simple->bounds, simple_data->is_static);
 
       cairo_save (cr);
       if (simple_data->transform)
@@ -1205,7 +1222,7 @@ goo_canvas_item_simple_update  (GooCanvasItem   *item,
       cairo_restore (cr);
 
       /* Request a redraw of the new bounds. */
-      goo_canvas_request_redraw (simple->canvas, &simple->bounds);
+      goo_canvas_request_item_redraw (simple->canvas, &simple->bounds, simple_data->is_static);
     }
 
   *bounds = simple->bounds;
@@ -1222,6 +1239,9 @@ goo_canvas_item_simple_get_requested_area (GooCanvasItem    *item,
   cairo_matrix_t matrix;
   double x_offset, y_offset;
 
+  /* Request a redraw of the existing bounds. */
+  goo_canvas_request_item_redraw (simple->canvas, &simple->bounds, simple_data->is_static);
+
   cairo_save (cr);
   if (simple_data->transform)
     cairo_transform (cr, simple_data->transform);
@@ -1290,6 +1310,7 @@ goo_canvas_item_simple_allocate_area      (GooCanvasItem         *item,
 					   gdouble                y_offset)
 {
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) item;
+  GooCanvasItemSimpleData *simple_data = simple->simple_data;
 
   /* Simple items can't resize at all, so we just adjust the bounds x & y
      positions here, and let the item be clipped if necessary. */
@@ -1297,6 +1318,9 @@ goo_canvas_item_simple_allocate_area      (GooCanvasItem         *item,
   simple->bounds.y1 += y_offset;
   simple->bounds.x2 += x_offset;
   simple->bounds.y2 += y_offset;
+
+  /* Request a redraw of the new bounds. */
+  goo_canvas_request_item_redraw (simple->canvas, &simple->bounds, simple_data->is_static);
 }
 
 
@@ -1333,21 +1357,31 @@ goo_canvas_item_simple_paint (GooCanvasItem         *item,
       cairo_clip (cr);
     }
 
-  if (class->simple_paint)
-    {
-      class->simple_paint (simple, cr, bounds);
-    }
-  else
-    {
-      class->simple_create_path (simple, cr);
-
-      goo_canvas_item_simple_paint_path (simple, cr);
-    }
+  class->simple_paint (simple, cr, bounds);
 
   cairo_restore (cr);
 }
 
 
+static void
+goo_canvas_item_simple_default_paint (GooCanvasItemSimple   *simple,
+				      cairo_t               *cr,
+				      const GooCanvasBounds *bounds)
+{
+  GooCanvasItemSimpleClass *class = GOO_CANVAS_ITEM_SIMPLE_GET_CLASS (simple);
+
+  class->simple_create_path (simple, cr);
+  goo_canvas_item_simple_paint_path (simple, cr);
+}
+
+
+static void
+goo_canvas_item_simple_default_create_path (GooCanvasItemSimple   *simple,
+					    cairo_t               *cr)
+{
+  /* Do nothing. */
+}
+
 
 static void
 goo_canvas_item_simple_title_changed (GooCanvasItemModelSimple *smodel,
@@ -1472,6 +1506,8 @@ canvas_item_interface_init (GooCanvasItemIface *iface)
   iface->get_style          = goo_canvas_item_simple_get_style;
   iface->set_style          = goo_canvas_item_simple_set_style;
   iface->is_visible         = goo_canvas_item_simple_is_visible;
+  iface->get_is_static	    = goo_canvas_item_simple_get_is_static;
+  iface->set_is_static	    = goo_canvas_item_simple_set_is_static;
 
   iface->get_model          = goo_canvas_item_simple_get_model;
   iface->set_model          = goo_canvas_item_simple_set_model_internal;
@@ -1897,17 +1933,12 @@ goo_canvas_item_model_simple_set_property (GObject              *object,
   GooCanvasItemSimpleData *simple_data = &smodel->simple_data;
   GooCanvasItemModel *parent;
   gboolean recompute_bounds;
-  gint child_num;
 
   switch (prop_id)
     {
     case PROP_PARENT:
       parent = g_value_get_object (value);
-      if (smodel->parent)
-	{
-	  child_num = goo_canvas_item_model_find_child (smodel->parent, model);
-	  goo_canvas_item_model_remove_child (smodel->parent, child_num);
-	}
+      goo_canvas_item_model_remove (model);
       goo_canvas_item_model_add_child (parent, model, -1);
       break;
     case PROP_TITLE:
