@@ -29,6 +29,7 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 #include "goocanvasrect.h"
+#include "goocanvas.h"
 
 
 enum {
@@ -44,17 +45,6 @@ enum {
 
 
 static void canvas_item_interface_init   (GooCanvasItemIface *iface);
-static void goo_canvas_rect_finalize     (GObject            *object);
-static void goo_canvas_rect_get_property (GObject            *object,
-					  guint               param_id,
-					  GValue             *value,
-					  GParamSpec         *pspec);
-static void goo_canvas_rect_set_property (GObject            *object,
-					  guint               param_id,
-					  const GValue       *value,
-					  GParamSpec         *pspec);
-static void goo_canvas_rect_create_path (GooCanvasItemSimple *simple,
-					 cairo_t             *cr);
 
 G_DEFINE_TYPE_WITH_CODE (GooCanvasRect, goo_canvas_rect,
 			 GOO_TYPE_CANVAS_ITEM_SIMPLE,
@@ -111,23 +101,6 @@ goo_canvas_rect_install_common_properties (GObjectClass *gobject_class)
 }
 
 
-static void
-goo_canvas_rect_class_init (GooCanvasRectClass *klass)
-{
-  GObjectClass *gobject_class = (GObjectClass*) klass;
-  GooCanvasItemSimpleClass *simple_class = (GooCanvasItemSimpleClass*) klass;
-
-  gobject_class->finalize     = goo_canvas_rect_finalize;
-
-  gobject_class->get_property = goo_canvas_rect_get_property;
-  gobject_class->set_property = goo_canvas_rect_set_property;
-
-  simple_class->simple_create_path = goo_canvas_rect_create_path;
-
-  goo_canvas_rect_install_common_properties (gobject_class);
-}
-
-
 static void
 goo_canvas_rect_init (GooCanvasRect *rect)
 {
@@ -210,7 +183,9 @@ goo_canvas_rect_finalize (GObject *object)
   GooCanvasItemSimple *simple = (GooCanvasItemSimple*) object;
   GooCanvasRect *rect = (GooCanvasRect*) object;
 
-  if (!simple->model)
+  /* Free our data if we didn't have a model. (If we had a model it would
+     have been reset in dispose() and simple_data will be NULL.) */
+  if (simple->simple_data)
     g_slice_free (GooCanvasRectData, rect->rect_data);
   rect->rect_data = NULL;
 
@@ -383,16 +358,30 @@ goo_canvas_rect_create_path (GooCanvasItemSimple *simple,
   else
     {
       /* Draw the plain rectangle. */
-      cairo_move_to (cr, rect_data->x, rect_data->y);
-      cairo_line_to (cr, rect_data->x, rect_data->y + rect_data->height);
-      cairo_line_to (cr, rect_data->x + rect_data->width,
-		     rect_data->y + rect_data->height);
-      cairo_line_to (cr, rect_data->x + rect_data->width, rect_data->y);
-      cairo_close_path (cr);
+      cairo_rectangle (cr, rect_data->x, rect_data->y,
+		       rect_data->width, rect_data->height);
     }
 }
 
 
+static void
+goo_canvas_rect_update  (GooCanvasItemSimple *simple,
+			 cairo_t             *cr)
+{
+  GooCanvasRect *rect = (GooCanvasRect*) simple;
+  GooCanvasRectData *rect_data = rect->rect_data;
+  gdouble half_line_width;
+
+  /* We can quickly compute the bounds as being just the rectangle's size
+     plus half the line width around each edge. */
+  half_line_width = goo_canvas_item_simple_get_line_width (simple) / 2;
+
+  simple->bounds.x1 = rect_data->x - half_line_width;
+  simple->bounds.y1 = rect_data->y - half_line_width;
+  simple->bounds.x2 = rect_data->x + rect_data->width + half_line_width;
+  simple->bounds.y2 = rect_data->y + rect_data->height + half_line_width;
+}
+
 
 static void
 goo_canvas_rect_set_model    (GooCanvasItem      *item,
@@ -421,6 +410,24 @@ canvas_item_interface_init (GooCanvasItemIface *iface)
 }
 
 
+static void
+goo_canvas_rect_class_init (GooCanvasRectClass *klass)
+{
+  GObjectClass *gobject_class = (GObjectClass*) klass;
+  GooCanvasItemSimpleClass *simple_class = (GooCanvasItemSimpleClass*) klass;
+
+  gobject_class->finalize     = goo_canvas_rect_finalize;
+
+  gobject_class->get_property = goo_canvas_rect_get_property;
+  gobject_class->set_property = goo_canvas_rect_set_property;
+
+  simple_class->simple_create_path = goo_canvas_rect_create_path;
+  simple_class->simple_update      = goo_canvas_rect_update;
+
+  goo_canvas_rect_install_common_properties (gobject_class);
+}
+
+
 /**
  * SECTION:goocanvasrectmodel
  * @Title: GooCanvasRectModel
